package com.nokia.finance.tasks.dao.house;

import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.annotations.Update;

@Mapper
public interface HouseProcedureDao {
    /**
     * 更新自有房产维修基本信息
     */
    @Update("""
update
    house.building_repair_month a
set
    site_id = b.site_id,
    site_name = b.site_name,
    site_num = b.site_num,
    second_unit = b.second_unit,
    third_unit = b.third_unit,
    city = b.city,
    district = b.district,
    area_no = b.area_no,
    area_name = b.area_name,
    city_no = b.city_no,
    city_name = b.city_name
from
    house.building_month b
where
    a.jz_jzid = b.building_id
    and a.year_month = b.year_month
    and a.year_month = #{endYearMonth}
""")
    int updateBuildingRepairMonthBaseInfo(@Param("endYearMonth") Integer endYearMonth);

    /**
     * 插入不动产自有房产相同维修频次统计
     * @param startYearMonth 开始年月
     * @param endYearMonth 结束年月
     */
    @Update("""
with
t101 as (
select
    year_month,
    jz_jzid,
    repair_type,
    content_category,
    form_no,
    sum(final_cost) as final_cost_sum
from
    house.building_repair_month
where repair_type != '财务预提冲销'
and year_month >= #{startYearMonth}
and year_month <= #{endYearMonth}
group by
    year_month,
    jz_jzid,
    repair_type,
    content_category,
    form_no
),
t102 as (
select
    jz_jzid,
    repair_type,
    content_category,
    count(1) as ty_total,
    sum(final_cost_sum) as final_cost_sum
from
    t101
group by
    jz_jzid,
    repair_type,
    content_category
),
t103 as (
select distinct on (building_id) * from house.building_month
where year_month >= #{startYearMonth} and year_month <= #{endYearMonth}
order by building_id, year_month desc
),
t104 as (
select
    t103.area_no,
    t103.area_name,
    t103.city_no,
    t103.city_name,
    t103.building_name,
    t103.site_id,
    t103.site_num,
    t103.site_name,
    t102.jz_jzid,
    t102.repair_type,
    t102.content_category,
    t102.ty_total,
    t102.final_cost_sum
from
    t102
join t103 on
    t102.jz_jzid = t103.building_id
),
t105 as (
select
        *,
        row_number() over (
order by
    ty_total desc, final_cost_sum desc
        ) as ty_total_rank
from
    t104
limit 20
),
t201 as (
select * from house.building_repair_month a
where year_month <= #{endYearMonth}
and repair_type != '财务预提冲销'
and exists (select 1 from t105 b where a.jz_jzid = b.jz_jzid)
),
t202 as (
select
    year_month,
    jz_jzid,
    repair_type,
    content_category,
    form_no
from
    t201
group by
    year_month,
    jz_jzid,
    repair_type,
    content_category,
    form_no
),
t203 as (
select
    jz_jzid,
    repair_type,
    content_category,
    count(1) as history_total
from
    t202
group by
    jz_jzid,
    repair_type,
    content_category
),
t301 as (
select
t105.*,
t203.history_total
from t105 left join t203
on t105.jz_jzid = t203.jz_jzid
and t105.repair_type = t203.repair_type
and t105.content_category = t203.content_category
)
insert into house.building_same_repair_frequency
(
year_month,
building_id,
area_no,
area_name,
city_no,
city_name,
building_name,
repair_type,
content_category,
ty_total,
ty_total_rank,
history_total,
final_cost_sum,
site_id,
site_num,
site_name
)
select
    #{endYearMonth} as year_month,
    jz_jzid as building_id,
    area_no,
    area_name,
    city_no,
    city_name,
    building_name,
    repair_type,
    content_category,
    ty_total,
    ty_total_rank,
    history_total,
    final_cost_sum,
    site_id,
    site_num,
    site_name
from
    t301
order by
    ty_total_rank
""")
    int insertBuildingSameRepairFrequency(@Param("startYearMonth") Integer startYearMonth,
                                          @Param("endYearMonth") Integer endYearMonth);

    /**
     * 插入不动产自有房产高总价维修统计
     * @param startYearMonth 开始年月
     * @param endYearMonth 结束年月
     */
    @Update("""
with
t101 as (
select
*
from
    house.building_repair_month
where
    repair_type != '财务预提冲销'
    and final_cost > 0
    and year_month >= #{startYearMonth}
    and year_month <= #{endYearMonth}
),
t102 as (
select distinct on (jz_jzid, repair_type, content_category) * from t101
order by jz_jzid, repair_type, content_category, year_month desc
),
t103 as (
select distinct on (building_id) * from house.building_month
where year_month >= #{startYearMonth} and year_month <= #{endYearMonth}
order by building_id, year_month desc
),
t104 as (
select
t102.year_month,
t102.jz_jzid,
t102.repair_type,
t102.content_category,
t103.site_id,
t103.site_num,
t103.site_name,
t103.area_no,
t103.area_name,
t103.city_no,
t103.city_name,
t103.building_name,
t103.address,
t103.building_use,
t103.building_area,
t103.building_area_self_use,
t103.building_area_rent,
t103.building_area_idle,
t103.building_area_unusable
from t102 join t103 on t102 .jz_jzid = t103.building_id
),
t105 as (
select
    jz_jzid,
    repair_type,
    content_category,
    sum(final_cost) as final_cost_sum
from
    t101
group by
    jz_jzid,
    repair_type,
    content_category
),
t106 as (
select
t105.final_cost_sum,
t104.*
from t105 join t104
on t105.jz_jzid = t104.jz_jzid
and t105.repair_type = t104.repair_type
and t105.content_category = t104.content_category
)
insert into house.building_high_repair
(
year_month,
building_id,
latest_repair_month,
area_no,
area_name,
city_no,
city_name,
building_name,
repair_type,
content_category,
final_cost_sum,
address,
building_use,
building_area,
building_area_self_use,
building_area_rent,
building_area_idle,
building_area_unusable,
site_id,
site_num,
site_name
)
select
#{endYearMonth} as year_month,
jz_jzid as building_id,
year_month as latest_repair_month,
area_no,
area_name,
city_no,
city_name,
building_name,
repair_type,
content_category,
final_cost_sum,
address,
building_use,
building_area,
building_area_self_use,
building_area_rent,
building_area_idle,
building_area_unusable,
site_id,
site_num,
site_name
from t106
order by final_cost_sum desc
limit 20
""")
    int insertBuildingHighRepair(@Param("startYearMonth") Integer startYearMonth,
                                 @Param("endYearMonth") Integer endYearMonth);

    /**
     * 插入不动产自有房产高额零星维修统计
     * @param startYearMonth 开始年月
     * @param endYearMonth 结束年月
     */
    @Update("""
with
t1 as (
select
    house.building_month.*,
    house.building_repair_month.year_month as repair_year_month,
    house.building_repair_month.repair_type,
    house.building_repair_month.content_category as repair_content,
    house.building_repair_month.final_cost as amount
from
    house.building_repair_month
join house.building_month on
    house.building_repair_month.jz_jzid = house.building_month.building_id
    and house.building_repair_month.year_month = house.building_month.year_month
where
    house.building_repair_month.repair_type = '零星维修'
    and house.building_repair_month.final_cost > 10000
    and house.building_repair_month.year_month >= #{startYearMonth}
    and house.building_repair_month.year_month <= #{endYearMonth}
),
t2 as (
select
building_id,
count(1) as total
from t1
group by building_id
),
t3 as (
select
t1.*,
t2.total
from t1 join t2 on t1.building_id = t2.building_id
)
insert into house.building_high_sporadic_repair
(
year_month,
building_id,
building_name,
repair_year_month,
repair_type,
repair_content,
amount,
total,
area_no,
area_name,
city_no,
city_name,
site_id,
site_num,
site_name
)
select
#{endYearMonth} as year_month,
building_id,
building_name,
repair_year_month,
repair_type,
repair_content,
amount,
total,
area_no,
area_name,
city_no,
city_name,
site_id,
site_num,
site_name
from t3
order by amount desc
""")
    int insertBuildingHighSporadicRepair(@Param("startYearMonth") Integer startYearMonth,
                                         @Param("endYearMonth") Integer endYearMonth);

    /**
     * 插入不动产自有房产闲置不可用发生维修
     * @param endYearMonth 账期
     */
    @Update("""
insert into house.building_idle_repair
(
year_month,
building_id,
building_name,
repair_type,
repair_content,
amount,
site_id,
site_name,
site_num,
area_no,
area_name,
city_no,
city_name,
address,
usage_status,
building_area,
building_area_self_use,
building_area_rent,
building_area_idle,
building_area_unusable
)
select
    house.building_repair_month.year_month,
    house.building_repair_month.jz_jzid as building_id,
    house.building_month.building_name,
    house.building_repair_month.repair_type,
    house.building_repair_month.content_category as repair_content,
    house.building_repair_month.final_cost as amount,
    house.building_month.site_id,
    house.building_month.site_name,
    house.building_month.site_num,
    house.building_month.area_no,
    house.building_month.area_name,
    house.building_month.city_no,
    house.building_month.city_name,
    house.building_month.address,
    house.building_month.usage_status,
    house.building_month.building_area,
    house.building_month.building_area_self_use,
    house.building_month.building_area_rent,
    house.building_month.building_area_idle,
    house.building_month.building_area_unusable
from
    house.building_repair_month
join house.building_month on
    house.building_repair_month.jz_jzid = house.building_month.building_id
    and house.building_repair_month.year_month = house.building_month.year_month
where
    house.building_repair_month.repair_type != '财务预提冲销'
    and house.building_repair_month.year_month = #{endYearMonth}
    and house.building_repair_month.final_cost > 0
    and house.building_month.building_area = (house.building_month.building_area_idle + house.building_month.building_area_unusable)
order by house.building_repair_month.final_cost desc
""")
    int insertBuildingIdleRepair(@Param("endYearMonth") Integer endYearMonth);
}
