package com.nokia.finance.tasks.dao.house;

import com.nokia.finance.tasks.pojo.po.house_report.HouseBuildingHighSporadicRepairPo;
import com.nokia.finance.tasks.pojo.po.house_report.HouseBuildingIdleStrategyPo;
import com.nokia.finance.tasks.pojo.po.house_report.HouseBuildingSameRepairFrequencyPo;
import com.nokia.finance.tasks.pojo.po.house_report.HouseBuildingStatPo;
import com.nokia.finance.tasks.pojo.po.house_report.HouseFangWuWeiXiuZongTiQingKuangPo;
import com.nokia.finance.tasks.pojo.po.house_report.HouseNoticesPo;
import com.nokia.finance.tasks.pojo.po.house_report.HouseRentOutIncomeStatPo;
import com.nokia.finance.tasks.pojo.po.house_report.HouseReportsPo;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.annotations.Select;
import org.apache.ibatis.annotations.Update;

import java.util.List;

@Mapper
public interface HouseReportDao {
    /**
     * 不动产建筑统计
     * @param endYearMonth 账期
     */
    @Select("""
with
t101 as (
select
    area_no,
    count(1) as fcsl,
    sum(building_area) as zmj,
    sum(case when area_sector = '核心商圈' then building_area else 0 end) as hxsq,
    sum(case when area_sector not in ('核心商圈', '乡镇', '乡村') then building_area else 0 end) as ybsq,
    sum(case when area_sector in ('乡镇', '乡村') then building_area else 0 end) as xzxc,
    sum(building_area_self_use) as zymj,
    sum(building_area_rent) as czmj,
    sum(building_area_idle) as xzmj,
    case
        when (sum(building_area_idle) + sum(building_area_rent)) = 0 then 0
        else round(sum(building_area_idle) / (sum(building_area_idle) + sum(building_area_rent)),
        4)
    end as xzl
from
    house.building_month
where
    year_month = #{endYearMonth}
group by
    area_no
),
t102 as (
select
    *,
    row_number() over (
    order by zmj desc) as zmj_rank
from
    t101
),
t103 as (
select
    a.id as area_no,
    a.name as area_name,
    b.fcsl,
    b.zmj,
    b.hxsq,
    b.ybsq,
    b.xzxc,
    b.zymj,
    b.czmj,
    b.xzmj,
    b.xzl,
    b.zmj_rank
from
    common.organization a
left join t102 b on
    a.id = b.area_no
where
    a.grade = 1
    and a.unhide = 1
order by
    a.order_num
),
t104 as (
select
    '018' as area_no,
    '合计' as area_name,
    sum(fcsl) as fcsl,
    sum(zmj) as zmj,
    sum(hxsq) as hxsq,
    sum(ybsq) as ybsq,
    sum(xzxc) as xzxc,
    sum(zymj) as zymj,
    sum(czmj) as czmj,
    sum(xzmj) as xzmj,
    case
        when (sum(xzmj) + sum(czmj)) = 0 then 0
        else round(sum(xzmj) / (sum(xzmj) + sum(czmj)),
        4)
    end as xzl,
    0 as zmj_rank
from
    t103
),
t201 as (
select
    *
from
    t103
union all
select
    *
from
    t104
)
select
    area_no,
    area_name,
    coalesce(fcsl,
    0) as fcsl,
    coalesce(zmj,
    0) as zmj,
    coalesce(hxsq,
    0) as hxsq,
    coalesce(ybsq,
    0) as ybsq,
    coalesce(xzxc,
    0) as xzxc,
    coalesce(zymj,
    0) as zymj,
    coalesce(czmj,
    0) as czmj,
    coalesce(xzmj,
    0) as xzmj,
    coalesce(xzl,
    0) as xzl,
    case
        when area_name = '石家庄' then 58339
        when area_name = '唐山' then 21675
        when area_name = '秦皇岛' then 19899
        when area_name = '邯郸' then 35219
        when area_name = '邢台' then 24820
        when area_name = '保定' then 36923
        when area_name = '张家口' then 21363
        when area_name = '承德' then 22220
        when area_name = '廊坊' then 20388
        when area_name = '沧州' then 31558
        when area_name = '衡水' then 16117
        when area_name = '雄安' then 3582
        when area_name = '省机动局' then 168516
        when area_name = '省本部' then 30039
        when area_name = '合计' then 510658
    end as zcyz
from
    t201
""")
    List<HouseBuildingStatPo> getBuildingStat(@Param("endYearMonth") Integer endYearMonth);

    /**
     * 闲置建筑面积超过1000平米策略
     * @param endYearMonth 账期
     */
    @Select("""
select
    *
from
    house.building_idle_strategy
where
    year_month = #{endYearMonth}
order by
    building_area_idle desc
""")
    List<HouseBuildingIdleStrategyPo> getBuildingIdleStrategy(@Param("endYearMonth") Integer endYearMonth);

    /**
     * 自有房产高额零星维修统计
     * @param endYearMonth 账期
     */
    @Select("""
select
    *
from
    house.building_high_sporadic_repair
where
    year_month = #{endYearMonth}
order by
    amount desc
limit 10
""")
    List<HouseBuildingHighSporadicRepairPo> getBuildingHighSporadicRepair(@Param("endYearMonth") Integer endYearMonth);

    /**
     * 不动产自有房产相同维修频次统计
     * @param endYearMonth 账期
     */
    @Select("""
select
    *
from
    house.building_same_repair_frequency
where
    year_month = #{endYearMonth}
order by
    ty_total_rank
""")
    List<HouseBuildingSameRepairFrequencyPo> getBuildingSameRepairFrequency(@Param("endYearMonth") Integer endYearMonth);

    /**
     * 对外出租收入统计
     * @param endYearMonth 账期
     */
    @Select("""
with
t101 as (
select
    area_name,
    income_all,
    income_ss,
    income_cx,
    budget_all,
    budget_ss,
    budget_cx,
    budget_finish_rate,
    last_year_income_all,
    yoy,
    yoy_percent,
    contribution_rate,
    budget_finish_rank,
    increase_amplitude_rank,
    sequential_progress,
    sequential_progress_diff
from
    house.rent_out_income_stat
where
    year_month = #{endYearMonth}
    and city_no = ''
order by
    area_no
),
t102 as (
select
    '全省' as area_name,
    sum(income_all) as income_all,
    sum(income_ss) as income_ss,
    sum(income_cx) as income_cx,
    sum(budget_all) as budget_all,
    sum(budget_ss) as budget_ss,
    sum(budget_cx) as budget_cx,
    case
        when sum(budget_all) = 0 then 0
        else round(sum(income_all) / sum(budget_all),
        4)
    end as budget_finish_rate,
    sum(last_year_income_all) as last_year_income_all,
    sum(yoy) as yoy,
    case
        when sum(last_year_income_all) = 0 then 0
        else round(sum(yoy) / sum(last_year_income_all),
        4)
    end as yoy_percent,
    1 as contribution_rate,
    null::int4 as budget_finish_rank,
    null::int4 as increase_amplitude_rank,
    max(sequential_progress) as sequential_progress
from
    t101
),
t103 as (
select
    area_name,
    income_all,
    income_ss,
    income_cx,
    budget_all,
    budget_ss,
    budget_cx,
    budget_finish_rate,
    last_year_income_all,
    yoy,
    yoy_percent,
    contribution_rate,
    budget_finish_rank,
    increase_amplitude_rank,
    sequential_progress,
    budget_finish_rate - sequential_progress as sequential_progress_diff
from
    t102
),
t104 as (
select
    *
from
    t101
union all
select
    *
from
    t103
)
select
    area_name,
    coalesce(income_all,
    0) as income_all,
    coalesce(income_ss,
    0) as income_ss,
    coalesce(income_cx,
    0) as income_cx,
    coalesce(budget_all,
    0) as budget_all,
    coalesce(budget_ss,
    0) as budget_ss,
    coalesce(budget_cx,
    0) as budget_cx,
    coalesce(budget_finish_rate,
    0) as budget_finish_rate,
    coalesce(last_year_income_all,
    0) as last_year_income_all,
    coalesce(yoy,
    0) as yoy,
    coalesce(yoy_percent,
    0) as yoy_percent,
    coalesce(contribution_rate,
    0) as contribution_rate,
    budget_finish_rank,
    increase_amplitude_rank,
    budget_finish_rank,
    increase_amplitude_rank,
    coalesce(sequential_progress,
    0) as sequential_progress,
    coalesce(sequential_progress_diff,
    0) as sequential_progress_diff
from
    t104
""")
    List<HouseRentOutIncomeStatPo> getRentOutIncomeStat(@Param("endYearMonth") Integer endYearMonth);

    /**
     * 房屋维修总体情况
     * @param endYearMonth 账期
     */
    @Select("""
select
    *
from
    house.fang_wu_wei_xiu_zong_ti_qing_kuang
where
    year_month = #{endYearMonth}
""")
    List<HouseFangWuWeiXiuZongTiQingKuangPo> fwwxztqk(@Param("endYearMonth") Integer endYearMonth);

    /**
     * 批量插入报告url
     */
    @Update("""
<script>
insert into house.house_reports
(
year_month,
file_type,
url
)
values
<foreach collection="list" item="item" index="index" separator=",">
(
#{item.yearMonth},
#{item.fileType},
#{item.url}
)
</foreach>
</script>
""")
    int insertBatchHouseReports(List<HouseReportsPo> list);

    /**
     * 批量插入公告
     */
    @Update("""
<script>
insert into house.notices
(
year_month,
city,
content,
create_time
)
values
<foreach collection="list" item="item" index="index" separator=",">
(
#{item.yearMonth},
#{item.city},
#{item.content},
#{item.createTime}
)
</foreach>
</script>
""")
    int insertBatchHouseNotices(List<HouseNoticesPo> list);

    /**
     * 删除公告
     * @param endYearMonth 账期
     */
    @Update("""
delete from house.notices where year_month = #{endYearMonth}
""")
    int deleteHouseNotices(@Param("endYearMonth") Integer endYearMonth);

    /**
     * 删除报告
     * @param endYearMonth 账期
     */
    @Update("""
delete from house.house_reports where year_month = #{endYearMonth}
""")
    int deleteHouseReports(@Param("endYearMonth") Integer endYearMonth);
}
