package com.nokia.finance.tasks.jobs.car;

import com.nokia.finance.tasks.common.exception.MyRuntimeException;
import com.nokia.finance.tasks.config.CarReportConfig;
import com.nokia.finance.tasks.dao.car.CarReportDao;
import com.nokia.finance.tasks.pojo.po.car_report.CarBglryQsPo;
import com.nokia.finance.tasks.pojo.po.car_report.CarBglryUnitPo;
import com.nokia.finance.tasks.pojo.po.car_report.CarBszsbPo;
import com.nokia.finance.tasks.pojo.po.car_report.CarClgkPo;
import com.nokia.finance.tasks.pojo.po.car_report.CarCqclqkPo;
import com.nokia.finance.tasks.pojo.po.car_report.CarDxPo;
import com.nokia.finance.tasks.pojo.po.car_report.CarDxUnitPo;
import com.nokia.finance.tasks.pojo.po.car_report.CarJygxPo;
import com.nokia.finance.tasks.pojo.po.car_report.CarJygxUnitPo;
import com.nokia.finance.tasks.pojo.po.car_report.CarNoticesPo;
import com.nokia.finance.tasks.pojo.po.car_report.CarReportsPo;
import com.nokia.finance.tasks.pojo.po.car_report.CarRyycPo;
import com.nokia.finance.tasks.pojo.po.car_report.CarRyycUnitPo;
import com.nokia.finance.tasks.pojo.po.car_report.CarSnfyPo;
import com.nokia.finance.tasks.pojo.po.car_report.CarWgPo;
import com.nokia.finance.tasks.pojo.po.car_report.CarWxQsPo;
import com.nokia.finance.tasks.pojo.po.car_report.CarWxUnitPo;
import com.nokia.finance.tasks.pojo.po.car_report.CarWzPo;
import com.nokia.finance.tasks.pojo.po.car_report.CarWzUnitPo;
import com.nokia.finance.tasks.pojo.po.car_report.CarZlwxycPo;
import com.nokia.finance.tasks.pojo.po.car_report.CarZywxycPo;
import com.nokia.finance.tasks.pojo.po.car_report.CarZywxycUnitPo;
import com.nokia.finance.tasks.pojo.vo.car_report.CarCldxVo;
import com.nokia.finance.tasks.pojo.vo.car_report.CarClfyVo;
import com.nokia.finance.tasks.pojo.vo.car_report.CarClgkVo;
import com.nokia.finance.tasks.pojo.vo.car_report.CarDxUnitVo;
import com.nokia.finance.tasks.pojo.vo.car_report.CarReportVo;
import com.nokia.finance.tasks.pojo.vo.car_report.CarWgUnitVo;
import com.nokia.finance.tasks.pojo.vo.car_report.CarWgycVo;
import com.nokia.finance.tasks.pojo.vo.car_report.CarWzUnitVo;
import com.nokia.finance.tasks.pojo.vo.car_report.CarWzclycVo;
import com.nokia.finance.tasks.pojo.vo.car_report.CarYcryfVo;
import com.nokia.finance.tasks.pojo.vo.car_report.CarZlycwxVo;
import com.nokia.finance.tasks.pojo.vo.car_report.CarZyycwxVo;
import com.nokia.finance.tasks.service.common.file.FileService;
import com.nokia.finance.tasks.utils.OfficeUtil;
import com.nokia.finance.tasks.utils.ZipUtil;
import freemarker.template.Configuration;
import freemarker.template.Template;
import freemarker.template.TemplateException;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.text.StringSubstitutor;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.io.BufferedWriter;
import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 车辆报告定时任务
 */
@Slf4j
@Service
public class CarReportJob {
    private final CarReportConfig carReportConfig;
    private final CarReportDao carReportDao;
    private final FileService fileService;

    public CarReportJob(CarReportConfig carReportConfig, CarReportDao carReportDao, FileService fileService) {
        this.carReportConfig = carReportConfig;
        this.carReportDao = carReportDao;
        this.fileService = fileService;
    }

    /**
     * 执行任务
     */
    @Scheduled(cron = "0 30 0 13 * ?")
    @Transactional(timeout = 300, rollbackFor = Exception.class)
    public void runJob() {
        try {
            log.info("执行车辆报告定时任务");
            CarReportVo vo = getReportVo();
            // 生成docx
            generateDocx(vo);
            // 生成pdf
            OfficeUtil.toPdf(carReportConfig.getOutDir(), carReportConfig.getDocx(), 1L);
            // 上传文件
            upload(vo);
            // 更新公告
            updateNotices(vo);
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            throw new MyRuntimeException("车辆报告定时任务失败");
        }
    }

    public CarReportVo getReportVo() throws IOException {
        LocalDate localDateNow = LocalDate.now();
        LocalDate localDateEnd = localDateNow.minusMonths(1);
        LocalDate localDateStart = localDateEnd.withDayOfYear(1);
        LocalDate localDateLastYear = localDateEnd.minusYears(1);
        Integer endYearMonth = Integer.valueOf(localDateEnd.format(DateTimeFormatter.ofPattern("yyyyMM")));
        Integer startYearMonth = Integer.valueOf(localDateStart.format(DateTimeFormatter.ofPattern("yyyyMM")));
        Integer yearMonthLastYear = Integer.valueOf(localDateLastYear.format(DateTimeFormatter.ofPattern("yyyyMM")));
        Integer endMonth = localDateEnd.getMonthValue();
        CarReportVo vo = new CarReportVo();
        vo.setReportYear(String.valueOf(localDateEnd.getYear()));
        vo.setReportMonth(String.valueOf(endMonth));
        vo.setCurrentYear(String.valueOf(localDateNow.getYear()));
        vo.setCurrentMonth(String.valueOf(localDateNow.getMonthValue()));
        vo.setLastReportYear(String.valueOf(localDateLastYear.getYear()));
        vo.setEndYearMonth(endYearMonth);
        // 车辆概况
        CarClgkPo carClgkPo = carReportDao.clgk(endYearMonth);
        // 产权车龄情况
        List<CarCqclqkPo> carCqclqkPoList = carReportDao.cqclqk(endYearMonth);
        // 建议更新车辆一级单位统计
        List<CarJygxUnitPo> carJygxUnitPoList = carReportDao.jygxUnit(startYearMonth, endYearMonth);
        // 建议更新车辆明细
        List<CarJygxPo> carJygxPoList = carReportDao.jygx(startYearMonth, endYearMonth);
        vo.setClgk(new CarClgkVo(carClgkPo, carCqclqkPoList, carJygxUnitPoList, carJygxPoList, carReportConfig));
        // 省内费用
        List<CarSnfyPo> carSnfyPoList = carReportDao.snfy(endYearMonth);
        // 北十占收比
        List<CarBszsbPo> carBszsbPoList = carReportDao.bszsb(endYearMonth);
        // 1-12月百公里燃油费变化情况
        List<CarBglryQsPo> carBglryQsPoList = carReportDao.bglryQs(startYearMonth, endYearMonth);
        // 二级组织机构百公里燃油费
        List<CarBglryUnitPo> carBglryUnitPoList = carReportDao.bglryUnit(startYearMonth, endYearMonth);
        // 1-12月维修费变化情况
        List<CarWxQsPo> carWxQsPoList = carReportDao.wxQs(startYearMonth, endYearMonth);
        // 二级组织机构自有车辆单车维修费
        List<CarWxUnitPo> carWxUnitPoList = carReportDao.wxUnit(endYearMonth);
        vo.setClfy(new CarClfyVo(carSnfyPoList, carBszsbPoList, carBglryQsPoList, carBglryUnitPoList, carWxQsPoList,
                carWxUnitPoList));
        // 异常燃油费一级单位统计
        List<CarRyycUnitPo> carRyycUnitPoList = carReportDao.ryycUnit(endYearMonth);
        // 异常燃油费明细
        List<CarRyycPo> carRyycPoList = carReportDao.ryyc(endYearMonth);
        vo.setYcryf(new CarYcryfVo(carRyycUnitPoList, carRyycPoList, carReportConfig));
        // 自有车辆维修费异常一级单位统计
        List<CarZywxycUnitPo> carZywxycUnitPoList = carReportDao.zywxycUnit(endYearMonth);
        // 自有车辆维修费异常
        List<CarZywxycPo> carZywxycPoList = carReportDao.zywxyc(endYearMonth);
        vo.setZyycwx(new CarZyycwxVo(carZywxycUnitPoList, carZywxycPoList, carReportConfig));
        // 租赁车辆维修费异常
        List<CarZlwxycPo> carZlwxycPoList = carReportDao.zlwxyc(endYearMonth);
        vo.setZlycwx(new CarZlycwxVo(carZlwxycPoList));
        // 低效车辆一级单位统计
        List<CarDxUnitPo> carDxUnitPoList = carReportDao.dxUnit(endYearMonth, yearMonthLastYear);
        // 低效车辆明细
        List<CarDxPo> carDxPoList = carReportDao.dx(endYearMonth);
        vo.setCldx(new CarCldxVo(carDxUnitPoList, carDxPoList, carReportConfig));
        // 违规用车一级单位统计
        List<CarWgUnitVo> carWgUnitVoList = carReportDao.wgUnit(startYearMonth, endYearMonth, endMonth);
        // 违规用车明细
        List<CarWgPo> carWgPoList = carReportDao.wg(endYearMonth);
        vo.setWgyc(new CarWgycVo(carWgUnitVoList, carWgPoList, carReportConfig));
        // 违章长期未处理一级单位统计
        List<CarWzUnitPo> carWzUnitPoList = carReportDao.wzUnit(endYearMonth);
        // 违章长期未处理明细
        List<CarWzPo> carWzPoList = carReportDao.wz(endYearMonth);
        vo.setWzclyc(new CarWzclycVo(carWzUnitPoList, carWzPoList, carReportConfig));
        return vo;
    }

    /**
     * 生成docx
     */
    public void generateDocx(CarReportVo vo) throws IOException, TemplateException {
        Configuration configuration = new Configuration(Configuration.DEFAULT_INCOMPATIBLE_IMPROVEMENTS);
        configuration.setDefaultEncoding(StandardCharsets.UTF_8.name());
        configuration.setDirectoryForTemplateLoading(new File(carReportConfig.getFtlDir()));
        // 生成document.xml
        try (BufferedWriter writer = Files.newBufferedWriter(Paths.get(carReportConfig.getDocumentXml()),
                StandardCharsets.UTF_8)) {
            Template template = configuration.getTemplate(carReportConfig.getDocumentFtl(),
                    StandardCharsets.UTF_8.name());
            template.process(vo, writer);
        }
        // 生成北十占收比图表xml
        try (BufferedWriter writer = Files.newBufferedWriter(Paths.get(carReportConfig.getBszsbChartXml()),
                StandardCharsets.UTF_8)) {
            Template template = configuration.getTemplate(carReportConfig.getBszsbChartFtl(),
                    StandardCharsets.UTF_8.name());
            template.process(vo, writer);
        }
        // 生成1-12月百公里燃油费变化情况图表xml
        try (BufferedWriter writer = Files.newBufferedWriter(Paths.get(carReportConfig.getBglryQsChartXml()),
                StandardCharsets.UTF_8)) {
            Template template = configuration.getTemplate(carReportConfig.getBglryQsChartFtl(),
                    StandardCharsets.UTF_8.name());
            template.process(vo, writer);
        }
        // 生成二级组织机构百公里燃油图表xml
        try (BufferedWriter writer = Files.newBufferedWriter(Paths.get(carReportConfig.getBglryUnitChartXml()),
                StandardCharsets.UTF_8)) {
            Template template = configuration.getTemplate(carReportConfig.getBglryUnitChartFtl(),
                    StandardCharsets.UTF_8.name());
            template.process(vo, writer);
        }
        // 生成1-12月维修费变化情况图表xml
        try (BufferedWriter writer = Files.newBufferedWriter(Paths.get(carReportConfig.getWxQsChartXml()),
                StandardCharsets.UTF_8)) {
            Template template = configuration.getTemplate(carReportConfig.getWxQsChartFtl(),
                    StandardCharsets.UTF_8.name());
            template.process(vo, writer);
        }
        // 生成二级组织机构自有车辆单车维修费图表xml
        try (BufferedWriter writer = Files.newBufferedWriter(Paths.get(carReportConfig.getWxUnitChartXml()),
                StandardCharsets.UTF_8)) {
            Template template = configuration.getTemplate(carReportConfig.getWxUnitChartFtl(),
                    StandardCharsets.UTF_8.name());
            template.process(vo, writer);
        }
        Files.createDirectories(Paths.get(carReportConfig.getOutDir()));
        Path inputDir = Paths.get(carReportConfig.getDocxDir());
        Path outputFile = Paths.get(carReportConfig.getDocx());
        ZipUtil.zipDirNonSelf(inputDir, outputFile);
    }

    /**
     * 上传文件
     */
    public void upload(CarReportVo vo) throws IOException {
        Integer endYearMonth = vo.getEndYearMonth();
        carReportDao.deleteCarReports(endYearMonth);
        List<CarReportsPo> list = new ArrayList<>();
        byte[] bytes = Files.readAllBytes(Paths.get(carReportConfig.getDocx()));
        long objectSize = bytes.length;
        String object = "oss/reports/car/" + endYearMonth + ".docx";
        fileService.putObject(object, new ByteArrayInputStream(bytes), objectSize, null);
        CarReportsPo po1 = new CarReportsPo();
        po1.setYearMonth(endYearMonth);
        po1.setFileType("word");
        po1.setUrl(object);
        list.add(po1);
        bytes = Files.readAllBytes(Paths.get(carReportConfig.getPdf()));
        objectSize = bytes.length;
        object = "oss/reports/car/" + endYearMonth + ".pdf";
        fileService.putObject(object, new ByteArrayInputStream(bytes), objectSize, null);
        CarReportsPo po2 = new CarReportsPo();
        po2.setYearMonth(endYearMonth);
        po2.setFileType("pdf");
        po2.setUrl(object);
        list.add(po2);
        int updateCount = carReportDao.insertBatchCarReports(list);
        if (updateCount == 0) {
            throw new MyRuntimeException("插入car.car_reports失败");
        }
    }

    /**
     * 更新公告
     */
    public void updateNotices(CarReportVo vo) {
        Integer endYearMonth = vo.getEndYearMonth();
        carReportDao.deleteCarNotices(endYearMonth);
        List<CarNoticesPo> list = new ArrayList<>();
        Map<String, String> map = new HashMap<>();
        StringSubstitutor substitutor = new StringSubstitutor(map);
        String template = "全省累计发生车辆费用${qsfy}万元，同比${tb}万元，百分比为${tbbfb}，车辆费用占收比为${zsb}，北十排名第${bspm}，占收比较${zsbgd}。省内${zsbjgdw}分公司占收比较高。";
        map.put("qsfy", vo.getClfy().getQsfy());
        map.put("tb", vo.getClfy().getTb());
        map.put("tbbfb", vo.getClfy().getTbbfb());
        map.put("zsb", vo.getClfy().getZsb());
        map.put("bspm", vo.getClfy().getBspm());
        map.put("zsbgd", vo.getClfy().getZsbgd());
        map.put("zsbjgdw", vo.getClfy().getZsbjgdw());
        list.add(new CarNoticesPo(endYearMonth, substitutor.replace(template)));
        map.clear();
        template = "全省车辆违规行驶累计${qswg}次，其中${wgzddw}违规车辆次数最多，${wgzdcs}次。";
        map.put("qswg", vo.getWgyc().getQswg());
        map.put("wgzddw", vo.getWgyc().getWgzddw());
        map.put("wgzdcs", vo.getWgyc().getWgzdcs());
        list.add(new CarNoticesPo(endYearMonth, substitutor.replace(template)));
        map.clear();
        for (CarDxUnitVo t : vo.getCldx().getDxUnitList()) {
            if ("合计".equals(t.getFirstUnit())) {
                continue;
            }
            template = "${firstUnit}低效车占比${diXiaoPercentTy}。";
            map.put("firstUnit", t.getFirstUnit());
            map.put("diXiaoPercentTy", t.getDiXiaoPercentTy());
            list.add(new CarNoticesPo(endYearMonth, t.getFirstUnit(), substitutor.replace(template)));
            map.clear();
        }
        for (CarWzUnitVo t : vo.getWzclyc().getWzUnitList()) {
            if ("总计".equals(t.getFirstUnit())) {
                continue;
            }
            template = "${firstUnit}超150天未处理违章${total}条";
            map.put("firstUnit", t.getFirstUnit());
            map.put("total", t.getTotal());
            list.add(new CarNoticesPo(endYearMonth, t.getFirstUnit(), substitutor.replace(template)));
            map.clear();
        }
        int updateCount = carReportDao.insertBatchCarNotices(list);
        if (updateCount == 0) {
            throw new MyRuntimeException("插入car.notices失败");
        }
    }
}
