package com.nokia.finance.tasks.jobs.car.procedure;

import com.nokia.finance.tasks.common.exception.MyRuntimeException;
import com.nokia.finance.tasks.dao.car.CarProcedureDao;
import com.nokia.finance.tasks.dao.gdc.car.GdcCarProcedureDao;
import lombok.extern.slf4j.Slf4j;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDate;
import java.time.format.DateTimeFormatter;

/**
 * 车辆基本信息月数据加工定时任务
 */
@Slf4j
@Service
public class CarBaseDataMonthProcJob {
    private final CarProcedureDao carProcedureDao;
    private final GdcCarProcedureDao gdcCarProcedureDao;

    public CarBaseDataMonthProcJob(CarProcedureDao carProcedureDao, GdcCarProcedureDao gdcCarProcedureDao) {
        this.carProcedureDao = carProcedureDao;
        this.gdcCarProcedureDao = gdcCarProcedureDao;
    }

    /**
     * 执行任务
     */
    @Scheduled(cron = "0 0 0 7 * ?")
    @Transactional(timeout = 60, rollbackFor = Exception.class)
    public void runJob() {
        try {
            log.info("执行车辆基本信息月数据加工定时任务");
            LocalDate localDateNow = LocalDate.now();
            LocalDate localDateEnd = localDateNow.minusMonths(1);
            Integer endYearMonth = Integer.valueOf(localDateEnd.format(DateTimeFormatter.ofPattern("yyyyMM")));
            int update1 = carProcedureDao.insertCarBaseDataMonth(endYearMonth);
            int update2 = carProcedureDao.updateCarBaseDataMonthChange(endYearMonth);
            int update3 = gdcCarProcedureDao.insertCarInfo(endYearMonth);
            gdcCarProcedureDao.insertCarScrap(endYearMonth);
            if (update1 == 0 || update2 == 0 || update3 == 0) {
                throw new MyRuntimeException("车辆基本信息月数据加工定时任务失败");
            }
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            throw new MyRuntimeException("车辆基本信息月数据加工定时任务失败");
        }
    }
}
