package com.nokia.finance.tasks.jobs.car.procedure;

import com.nokia.finance.tasks.common.exception.MyRuntimeException;
import com.nokia.finance.tasks.dao.car.CarProcedureDao;
import com.nokia.finance.tasks.dao.gdc.car.GdcCarProcedureDao;
import lombok.extern.slf4j.Slf4j;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDate;
import java.time.format.DateTimeFormatter;

/**
 * 车辆费用数据加工定时任务
 */
@Slf4j
@Service
public class CarFeeProcJob {
    private final CarProcedureDao carProcedureDao;
    private final GdcCarProcedureDao gdcCarProcedureDao;

    public CarFeeProcJob(CarProcedureDao carProcedureDao, GdcCarProcedureDao gdcCarProcedureDao) {
        this.carProcedureDao = carProcedureDao;
        this.gdcCarProcedureDao = gdcCarProcedureDao;
    }

    /**
     * 执行任务
     */
    @Scheduled(cron = "0 2 0 13 * ?")
    @Transactional(timeout = 60, rollbackFor = Exception.class)
    public void runJob() {
        try {
            log.info("执行车辆费用数据加工定时任务");
            // 当前日期
            LocalDate localDateNow = LocalDate.now();
            // 上个月1号
            LocalDate localDateEnd = localDateNow.minusMonths(1);
            // 年初1月1号
            LocalDate localDateStart = localDateEnd.withDayOfYear(1);
            // 结束年月
            Integer endYearMonth = Integer.valueOf(localDateEnd.format(DateTimeFormatter.ofPattern("yyyyMM")));
            // 开始年月
            Integer startYearMonth = Integer.valueOf(localDateStart.format(DateTimeFormatter.ofPattern("yyyyMM")));
            carProcedureDao.insertCarQiTa(endYearMonth);
            carProcedureDao.insertCarNianJianFei(endYearMonth);
            carProcedureDao.insertCarBaoXian(endYearMonth);
            carProcedureDao.insertCarLuQiao(endYearMonth);
            carProcedureDao.insertCarWeiXiu(endYearMonth);
            carProcedureDao.insertCarRanYou(endYearMonth);
            carProcedureDao.insertCarZuLin(endYearMonth);
            carProcedureDao.updateCarQiTan(endYearMonth);
            carProcedureDao.updateCarNianJianFei(endYearMonth);
            carProcedureDao.updateCarBaoXian(endYearMonth);
            carProcedureDao.updateCarLuQiao(endYearMonth);
            carProcedureDao.updateCarWeiXiu(endYearMonth);
            carProcedureDao.updateCarRanYou(endYearMonth);
            carProcedureDao.updateCarZuLin(endYearMonth);
            int update1 = carProcedureDao.updateCarFeiYongMonthYunXingFeiZongFeiYong(endYearMonth);
            int update2 = carProcedureDao.updateCarFeiYongMonthBaseInfo(endYearMonth);
            carProcedureDao.insertCarRanYouYiChang(startYearMonth, endYearMonth);
            carProcedureDao.insertCarZiYouWeiXiuYiChang(startYearMonth, endYearMonth);
            carProcedureDao.insertCarZuLinWeiXiuYiChang(startYearMonth, endYearMonth);
            int update3 = gdcCarProcedureDao.insertOneCarOneTable(endYearMonth);
            if (update1 == 0 || update2 == 0 || update3 == 0) {
                throw new MyRuntimeException("车辆费用数据加工定时任务失败");
            }
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            throw new MyRuntimeException("车辆费用数据加工定时任务失败");
        }
    }
}
