package com.nokia.finance.tasks.jobs.car.procedure;

import com.nokia.finance.tasks.common.exception.MyRuntimeException;
import com.nokia.finance.tasks.dao.car.CarProcedureDao;
import lombok.extern.slf4j.Slf4j;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDate;
import java.time.format.DateTimeFormatter;

/**
 * 车辆行驶里程月数据加工定时任务
 */
@Slf4j
@Service
public class CarLiChengMonthProcJob {
    private final CarProcedureDao carProcedureDao;

    public CarLiChengMonthProcJob(CarProcedureDao carProcedureDao) {
        this.carProcedureDao = carProcedureDao;
    }

    /**
     * 执行任务
     */
    @Scheduled(cron = "0 2 0 7 * ?")
    @Transactional(timeout = 60, rollbackFor = Exception.class)
    public void runJob() {
        try {
            log.info("执行车辆行驶里程月数据加工定时任务");
            // 当前日期
            LocalDate localDateNow = LocalDate.now();
            // 上个月1号
            LocalDate localDateEnd = localDateNow.minusMonths(1);
            // 年初1月1号
            LocalDate localDateStart = localDateEnd.withDayOfYear(1);
            // 结束年月
            Integer endYearMonth = Integer.valueOf(localDateEnd.format(DateTimeFormatter.ofPattern("yyyyMM")));
            // 开始年月
            Integer startYearMonth = Integer.valueOf(localDateStart.format(DateTimeFormatter.ofPattern("yyyyMM")));
            int update1 = carProcedureDao.updateCarLiChengMonthWorkday(endYearMonth);
            int update2 = carProcedureDao.updateCarLiChengMonthRiJunLiChengChuQinLv(endYearMonth);
            int update3 = carProcedureDao.updateCarLiChengMonthBaseInfo(endYearMonth);
            int update4 = carProcedureDao.updateCarLiChengMonthDiXiao(endYearMonth);
            int update5 = carProcedureDao.insertCarLiChengLeiJiMonth(startYearMonth, endYearMonth);
            carProcedureDao.insertCarDiXiaoMonth(endYearMonth);
            if (update1 == 0 || update2 == 0 || update3 == 0 || update4 == 0 || update5 == 0) {
                throw new MyRuntimeException("车辆行驶里程月数据加工定时任务失败");
            }
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            throw new MyRuntimeException("车辆行驶里程月数据加工定时任务失败");
        }
    }
}
