package com.nokia.finance.tasks.jobs.car.procedure;

import com.nokia.finance.tasks.common.exception.MyRuntimeException;
import com.nokia.finance.tasks.dao.car.CarProcedureDao;
import com.nokia.finance.tasks.dao.gdc.car.GdcCarProcedureDao;
import lombok.extern.slf4j.Slf4j;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDate;
import java.time.format.DateTimeFormatter;

/**
 * 车辆派单数据加工定时任务
 */
@Slf4j
@Service
public class CarPaiDanProcJob {
    private final CarProcedureDao carProcedureDao;
    private final GdcCarProcedureDao gdcCarProcedureDao;

    public CarPaiDanProcJob(CarProcedureDao carProcedureDao, GdcCarProcedureDao gdcCarProcedureDao) {
        this.carProcedureDao = carProcedureDao;
        this.gdcCarProcedureDao = gdcCarProcedureDao;
    }

    /**
     * 执行任务
     */
    @Scheduled(cron = "0 4 0 7 * ?")
    @Transactional(timeout = 60, rollbackFor = Exception.class)
    public void runJob() {
        try {
            log.info("执行车辆派单数据加工定时任务");
            // 当前日期
            LocalDate localDateNow = LocalDate.now();
            // 上个月1号
            LocalDate localDateEnd = localDateNow.minusMonths(1);
            // 年初1月1号
            LocalDate localDateStart = localDateEnd.withDayOfYear(1);
            // 结束年月
            Integer endYearMonth = Integer.valueOf(localDateEnd.format(DateTimeFormatter.ofPattern("yyyyMM")));
            // 开始年月
            Integer startYearMonth = Integer.valueOf(localDateStart.format(DateTimeFormatter.ofPattern("yyyyMM")));
            boolean flag = carProcedureDao.hasCarBaseDataYongChe(endYearMonth);
            if (!flag) {
                throw new MyRuntimeException("没有车辆基本信息、用车数据");
            }
            carProcedureDao.insertCarWeiGui(endYearMonth);
            gdcCarProcedureDao.insertCarWeiGui(endYearMonth);
            gdcCarProcedureDao.insertCarWuDanYongCheBaoJing(endYearMonth);
            gdcCarProcedureDao.insertCarWuPaiDanYueJieTongJi(startYearMonth, endYearMonth);
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            throw new MyRuntimeException("车辆派单数据加工定时任务失败");
        }
    }
}
