package com.nokia.finance.tasks.jobs.house.procedure;

import com.nokia.finance.tasks.common.exception.MyRuntimeException;
import com.nokia.finance.tasks.dao.house.HouseProcedureDao;
import lombok.extern.slf4j.Slf4j;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDate;
import java.time.format.DateTimeFormatter;

/**
 * 不动产自有房产维修数据加工定时任务
 */
@Slf4j
@Service
public class HouseBuildingRepairProcJob {
    private final HouseProcedureDao houseProcedureDao;

    public HouseBuildingRepairProcJob(HouseProcedureDao houseProcedureDao) {
        this.houseProcedureDao = houseProcedureDao;
    }

    /**
     * 执行任务
     */
    @Scheduled(cron = "0 0 0 13 * ?")
    @Transactional(timeout = 60, rollbackFor = Exception.class)
    public void runJob() {
        try {
            log.info("执行不动产自有房产维修数据加工定时任务");
            // 当前日期
            LocalDate localDateNow = LocalDate.now();
            // 上个月1号
            LocalDate localDateEnd = localDateNow.minusMonths(1);
            // 年初1月1号
            LocalDate localDateStart = localDateEnd.withDayOfYear(1);
            // 结束年月
            Integer endYearMonth = Integer.valueOf(localDateEnd.format(DateTimeFormatter.ofPattern("yyyyMM")));
            // 开始年月
            Integer startYearMonth = Integer.valueOf(localDateStart.format(DateTimeFormatter.ofPattern("yyyyMM")));
            int update1 = houseProcedureDao.insertBuildingSameRepairFrequency(startYearMonth, endYearMonth);
            int update2 = houseProcedureDao.insertBuildingHighRepair(startYearMonth, endYearMonth);
            int update3 = houseProcedureDao.insertBuildingHighSporadicRepair(startYearMonth, endYearMonth);
            houseProcedureDao.insertBuildingIdleRepair(endYearMonth);
            if (update1 == 0 || update2 == 0 || update3 == 0) {
                throw new MyRuntimeException("不动产自有房产维修数据加工定时任务失败");
            }
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            throw new MyRuntimeException("不动产自有房产维修数据加工定时任务失败");
        }
    }
}
