package com.nokia.finance.tasks.pojo.vo.car_report;

import com.nokia.finance.tasks.pojo.po.car_report.CarBglryQsPo;
import com.nokia.finance.tasks.pojo.po.car_report.CarBglryUnitPo;
import com.nokia.finance.tasks.pojo.po.car_report.CarBszsbPo;
import com.nokia.finance.tasks.pojo.po.car_report.CarSnfyPo;
import com.nokia.finance.tasks.pojo.po.car_report.CarWxQsPo;
import com.nokia.finance.tasks.pojo.po.car_report.CarWxUnitPo;
import lombok.Data;
import org.springframework.util.CollectionUtils;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;
import java.util.stream.Collectors;
import java.util.stream.Stream;

@Data
public class CarClfyVo {
    /**
     * 全省累计发生车辆费用万元
     */
    private String qsfy;
    /**
     * 全省费用同比增减万元
     */
    private String tb;
    /**
     * 全省费用同比增减百分比
     */
    private String tbbfb;
    /**
     * 占收比
     */
    private String zsb;
    /**
     * 北十排名
     */
    private String bspm;
    /**
     * 占收比高低
     */
    private String zsbgd;
    /**
     * 占收比较高单位
     */
    private String zsbjgdw;
    /**
     * 省内车辆费用
     */
    private List<CarSnfyVo> snfyList;
    /**
     * 北十车辆费用占收比对标
     */
    private List<CarBszsbVo> bszsbList;
    /**
     * 1-12月百公里燃油费变化情况
     */
    private List<CarBglryQsVo> bglryQsList;
    /**
     * 二级组织机构百公里燃油
     */
    private List<CarBglryUnitVo> bglryUnitList;
    /**
     * 1-12月维修费变化情况
     */
    private List<CarWxQsVo> wxQsList;
    /**
     * 二级组织机构自有车辆单车维修费
     */
    private List<CarWxUnitVo> wxUnitList;

    public CarClfyVo(List<CarSnfyPo> snfyList, List<CarBszsbPo> bszsbList, List<CarBglryQsPo> bglryQsList,
                     List<CarBglryUnitPo> bglryUnitList, List<CarWxQsPo> wxQsList, List<CarWxUnitPo> wxUnitList) {
        this.snfyList = new ArrayList<>();
        this.bszsbList = new ArrayList<>();
        this.bglryQsList = new ArrayList<>();
        this.bglryUnitList = new ArrayList<>();
        this.wxQsList = new ArrayList<>();
        this.wxUnitList = new ArrayList<>();
        List<String> unitList = Stream.of("石家庄", "唐山", "秦皇岛", "邯郸", "邢台", "保定", "张家口", "承德",
                "廊坊", "沧州", "衡水", "雄安", "传输局", "省本部", "合计").toList();
        int unitListSize = unitList.size();
        int snfyListSize = snfyList.size();
        if (snfyListSize == 1) {
            this.qsfy = CarReportVo.PLACEHOLDER;
            this.tb = CarReportVo.PLACEHOLDER;
            this.tbbfb = CarReportVo.PLACEHOLDER;
            this.zsbjgdw = CarReportVo.PLACEHOLDER;
            for (int i = 0; i < unitListSize; i++) {
                String firstUnit = unitList.get(i);
                CarSnfyVo carSnfyVo = new CarSnfyVo(String.valueOf(i + 1), firstUnit);
                this.snfyList.add(carSnfyVo);
            }
        } else {
            DecimalFormat integerFormat = new DecimalFormat("#,###");
            for (int i = 0; i < unitListSize; i++) {
                String firstUnit = unitList.get(i);
                CarSnfyVo carSnfyVo = new CarSnfyVo(i + 1, firstUnit);
                this.snfyList.add(carSnfyVo);
                for (CarSnfyPo carSnfyPo : snfyList) {
                    if (firstUnit.equals(carSnfyPo.getSecondOrgName())) {
                        carSnfyVo.update(carSnfyPo);
                        if ("合计".equals(carSnfyPo.getSecondOrgName())) {
                            this.qsfy = integerFormat.format(carSnfyPo.getTyFy());
                            BigDecimal tbzj = carSnfyPo.getTbzj();
                            this.tb = (tbzj.compareTo(BigDecimal.ZERO) >= 0 ? "增加" : "减少")
                                    + integerFormat.format(tbzj.abs());
                            this.tbbfb = carSnfyVo.getTbbfb();
                        }
                        break;
                    }
                }
            }
            BigDecimal median;
            List<BigDecimal> l = snfyList.stream()
                    .filter(t -> !t.getSecondOrgName().equals("合计") && t.getTyZsb() != null)
                    .map(CarSnfyPo::getTyZsb).sorted().toList();
            int size = l.size();
            if (size % 2 == 0) {
                int midIndex = size / 2;
                BigDecimal firstMidValue = l.get(midIndex - 1);
                BigDecimal secondMidValue = l.get(midIndex);
                median = firstMidValue.add(secondMidValue).divide(new BigDecimal("2"), RoundingMode.HALF_UP);
            } else {
                int midIndex = size / 2;
                median = l.get(midIndex);
            }
            this.zsbjgdw = snfyList.stream()
                    .filter(t -> !t.getSecondOrgName().equals("合计")
                            && t.getTyZsb() != null
                            && t.getTyZsb().compareTo(median) > 0)
                    .sorted(Comparator.comparing(CarSnfyPo::getTyZsb).reversed())
                    .map(CarSnfyPo::getSecondOrgName).collect(Collectors.joining("、"));
        }
        if (CollectionUtils.isEmpty(bszsbList)) {
            this.zsb = CarReportVo.PLACEHOLDER;
            this.bspm = CarReportVo.PLACEHOLDER;
            this.zsbgd = CarReportVo.PLACEHOLDER;
        } else {
            int bszsbListSize = bszsbList.size();
            for (int i = 0; i < bszsbListSize; i++) {
                CarBszsbPo carBszsbPo = bszsbList.get(i);
                this.bszsbList.add(new CarBszsbVo(String.valueOf(i), carBszsbPo));
                if ("河北".equals(carBszsbPo.getAreaName())) {
                    DecimalFormat percentFormat = new DecimalFormat("0.000%");
                    this.zsb = percentFormat.format(carBszsbPo.getTyZsb());
                    this.bspm = carBszsbPo.getTyZsbRank().toString();
                    this.zsbgd = carBszsbPo.getTyZsbRank().compareTo(5) <= 0 ? "高" : "低";
                }
            }
        }
        for (CarBglryQsPo carBglryQsPo : bglryQsList) {
            this.bglryQsList.add(new CarBglryQsVo(carBglryQsPo));
        }
        int bglryUnitListSize = bglryUnitList.size();
        for (int i = 0; i < bglryUnitListSize; i++) {
            CarBglryUnitPo carBglryUnitPo = bglryUnitList.get(i);
            this.bglryUnitList.add(new CarBglryUnitVo(String.valueOf(i), carBglryUnitPo));
        }
        for (CarWxQsPo carWxQsPo : wxQsList) {
            this.wxQsList.add(new CarWxQsVo(carWxQsPo));
        }
        int wxUnitListSize = wxUnitList.size();
        for (int i = 0; i < wxUnitListSize; i++) {
            CarWxUnitPo carWxUnitPo = wxUnitList.get(i);
            this.wxUnitList.add(new CarWxUnitVo(String.valueOf(i), carWxUnitPo));
        }
    }
}
