package com.nokia.finance.tasks.pojo.vo.car_report;

import com.nokia.finance.tasks.config.CarReportConfig;
import com.nokia.finance.tasks.pojo.po.car_report.CarClgkPo;
import com.nokia.finance.tasks.pojo.po.car_report.CarCqclqkPo;
import com.nokia.finance.tasks.pojo.po.car_report.CarJygxPo;
import com.nokia.finance.tasks.pojo.po.car_report.CarJygxUnitPo;
import lombok.Data;
import lombok.NoArgsConstructor;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.CellStyle;
import org.apache.poi.ss.usermodel.DataFormat;
import org.apache.poi.ss.usermodel.HorizontalAlignment;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.VerticalAlignment;
import org.apache.poi.xssf.streaming.SXSSFSheet;
import org.apache.poi.xssf.streaming.SXSSFWorkbook;

import java.io.IOException;
import java.io.OutputStream;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.stream.Stream;

@Data
@NoArgsConstructor
public class CarClgkVo {
    /**
     * 车辆总数
     */
    private String total;
    /**
     * 自有车辆
     */
    private String ziYouCount;
    /**
     * 租赁车辆
     */
    private String zuLinCount;
    /**
     * 生产用车
     */
    private String shengChanCount;
    /**
     * 公务用车
     */
    private String gongWuCount;
    /**
     * 皮卡车
     */
    private String piKaCount;
    /**
     * 微型面包车
     */
    private String weiXingMianBaoCount;
    /**
     * suv
     */
    private String yueYeCount;
    /**
     * 轿车
     */
    private String jiaoCheCount;
    /**
     * 成新率
     */
    private String chengXinLv;
    /**
     * 产权车龄情况
     */
    private List<CarCqclqkVo> cqclqkList;
    /**
     * 建议更新车辆一级单位统计
     */
    private List<CarJygxUnitVo> jygxUnitList;

    public CarClgkVo(CarClgkPo carClgkPo, List<CarCqclqkPo> carCqclqkPoList, List<CarJygxUnitPo> carJygxUnitPoList,
                     List<CarJygxPo> carJygxPoList, CarReportConfig carReportConfig) throws IOException {
        this.cqclqkList = new ArrayList<>();
        this.jygxUnitList = new ArrayList<>();
        List<String> unitList = Stream.of("石家庄", "唐山", "秦皇岛", "邯郸", "邢台", "保定", "张家口", "承德", "廊坊",
                "沧州", "衡水", "雄安", "机动局", "省公司本部", "雄安基地建设部", "华北基地建设部", "合计").toList();
        int unitListSize = unitList.size();
        if (carClgkPo.getTotal().equals(0)) {
            this.total = CarReportVo.PLACEHOLDER;
            this.ziYouCount = CarReportVo.PLACEHOLDER;
            this.zuLinCount = CarReportVo.PLACEHOLDER;
            this.shengChanCount = CarReportVo.PLACEHOLDER;
            this.gongWuCount = CarReportVo.PLACEHOLDER;
            this.piKaCount = CarReportVo.PLACEHOLDER;
            this.weiXingMianBaoCount = CarReportVo.PLACEHOLDER;
            this.yueYeCount = CarReportVo.PLACEHOLDER;
            this.jiaoCheCount = CarReportVo.PLACEHOLDER;
            this.chengXinLv = CarReportVo.PLACEHOLDER;
            for (int i = 0; i < unitListSize; i++) {
                String firstUnit = unitList.get(i);
                CarCqclqkVo carCqclqkVo = new CarCqclqkVo(String.valueOf(i + 1), firstUnit);
                this.cqclqkList.add(carCqclqkVo);
            }
            this.jygxUnitList.add(new CarJygxUnitVo());
        } else {
            DecimalFormat integerFormat = new DecimalFormat("#,###");
            this.total = integerFormat.format(carClgkPo.getTotal());
            this.ziYouCount = integerFormat.format(carClgkPo.getZiYouCount());
            this.zuLinCount = integerFormat.format(carClgkPo.getZuLinCount());
            this.shengChanCount = integerFormat.format(carClgkPo.getShengChanCount());
            this.gongWuCount = integerFormat.format(carClgkPo.getGongWuCount());
            this.piKaCount = integerFormat.format(carClgkPo.getPiKaCount());
            this.weiXingMianBaoCount = integerFormat.format(carClgkPo.getWeiXingMianBaoCount());
            this.yueYeCount = integerFormat.format(carClgkPo.getYueYeCount());
            this.jiaoCheCount = integerFormat.format(carClgkPo.getJiaoCheCount());
            for (int i = 0; i < unitListSize; i++) {
                String firstUnit = unitList.get(i);
                CarCqclqkVo carCqclqkVo = new CarCqclqkVo(i + 1, firstUnit);
                this.cqclqkList.add(carCqclqkVo);
                for (CarCqclqkPo carCqclqkPo : carCqclqkPoList) {
                    if (firstUnit.equals(carCqclqkPo.getFirstUnit())) {
                        carCqclqkVo.update(carCqclqkPo);
                        break;
                    }
                }
                if ("合计".equals(firstUnit)) {
                    chengXinLv = carCqclqkVo.getChengXinLv();
                }
            }
            carJygxUnitPoList.forEach(t -> this.jygxUnitList.add(new CarJygxUnitVo(t)));
        }
        writeJygxXlsx(carJygxPoList, carReportConfig);
    }

    /**
     * 建议更新车辆明细xlsx
     */
    private void writeJygxXlsx(List<CarJygxPo> carJygxPoList, CarReportConfig carReportConfig) throws IOException {
        try (SXSSFWorkbook wb = new SXSSFWorkbook();
//             FastByteArrayOutputStream os = new FastByteArrayOutputStream()
             OutputStream os = Files.newOutputStream(Paths.get(carReportConfig.getJygxXlsx()))
        ) {
            DataFormat dataFormat = wb.createDataFormat();
            // 数字样式
            CellStyle numberCellStyle = wb.createCellStyle();
            numberCellStyle.setDataFormat(dataFormat.getFormat("#,##0.00"));
            numberCellStyle.setAlignment(HorizontalAlignment.RIGHT);
            // 整数样式
            CellStyle integerCellStyle = wb.createCellStyle();
            integerCellStyle.setDataFormat(dataFormat.getFormat("#,##0"));
            integerCellStyle.setAlignment(HorizontalAlignment.RIGHT);
            // 日期样式
            CellStyle dateCellStyle = wb.createCellStyle();
            dateCellStyle.setDataFormat(dataFormat.getFormat("yyyy/MM/dd"));
            // 表头样式
            CellStyle headerCellStyle = wb.createCellStyle();
            headerCellStyle.setAlignment(HorizontalAlignment.CENTER);
            headerCellStyle.setVerticalAlignment(VerticalAlignment.CENTER);
            SXSSFSheet sheet = wb.createSheet();
            AtomicInteger rowIndex = new AtomicInteger(0);
            // 表头
            Row headerRow = sheet.createRow(rowIndex.getAndIncrement());
            List<String> headers = Stream.of("车牌号", "车辆品牌", "登记日期", "车辆来源", "车辆类型",
                    "车辆使用性质", "车辆所属单位", "单位", "二级", "三级", "车龄", "维修累计").toList();
            int headerSize = headers.size();
            for (int i = 0; i < headerSize; i++) {
                Cell cell = headerRow.createCell(i);
                cell.setCellValue(headers.get(i));
                cell.setCellStyle(headerCellStyle);
                // 根据内容长度设置列宽
                int columnWidth = headers.get(i).length() * 256 * 2 + 256 * 4;
                sheet.setColumnWidth(i, columnWidth);
            }
            for (CarJygxPo t : carJygxPoList) {
                AtomicInteger columnIndex = new AtomicInteger(0);
                Row row = sheet.createRow(rowIndex.getAndIncrement());
                // 车牌号
                Cell chePaiHaoCell = row.createCell(columnIndex.getAndIncrement());
                if (t.getChePaiHao() != null) {
                    chePaiHaoCell.setCellValue(t.getChePaiHao());
                }
                // 车辆品牌
                Cell cheLiangPinPaiCell = row.createCell(columnIndex.getAndIncrement());
                if (t.getCheLiangPinPai() != null) {
                    cheLiangPinPaiCell.setCellValue(t.getCheLiangPinPai());
                }
                // 登记日期
                Cell dengJiRiQiCell = row.createCell(columnIndex.getAndIncrement());
                if (t.getDengJiRiQi() != null) {
                    dengJiRiQiCell.setCellValue(t.getDengJiRiQi());
                }
                dengJiRiQiCell.setCellStyle(dateCellStyle);
                // 车辆来源
                Cell cheLiangLaiYuanCell = row.createCell(columnIndex.getAndIncrement());
                if (t.getCheLiangLaiYuan() != null) {
                    cheLiangLaiYuanCell.setCellValue(t.getCheLiangLaiYuan());
                }
                // 车辆类型
                Cell cheLiangLeiXingCell = row.createCell(columnIndex.getAndIncrement());
                if (t.getCheLiangLeiXing() != null) {
                    cheLiangLeiXingCell.setCellValue(t.getCheLiangLeiXing());
                }
                // 车辆使用性质
                Cell cheLiangShiYongXingZhiCell = row.createCell(columnIndex.getAndIncrement());
                if (t.getCheLiangShiYongXingZhi() != null) {
                    cheLiangShiYongXingZhiCell.setCellValue(t.getCheLiangShiYongXingZhi());
                }
                // 车辆所属单位
                Cell cheLiangSuoShuDanWeiCell = row.createCell(columnIndex.getAndIncrement());
                if (t.getCheLiangSuoShuDanWei() != null) {
                    cheLiangSuoShuDanWeiCell.setCellValue(t.getCheLiangSuoShuDanWei());
                }
                // 单位
                Cell firstUnitCell = row.createCell(columnIndex.getAndIncrement());
                if (t.getFirstUnit() != null) {
                    firstUnitCell.setCellValue(t.getFirstUnit());
                }
                // 二级
                Cell secondUnitCell = row.createCell(columnIndex.getAndIncrement());
                if (t.getSecondUnit() != null) {
                    secondUnitCell.setCellValue(t.getSecondUnit());
                }
                // 三级
                Cell thirdUnitCell = row.createCell(columnIndex.getAndIncrement());
                if (t.getThirdUnit() != null) {
                    thirdUnitCell.setCellValue(t.getThirdUnit());
                }
                // 车龄
                Cell cheLingCell = row.createCell(columnIndex.getAndIncrement());
                if (t.getCheLing() != null) {
                    cheLingCell.setCellValue(t.getCheLing());
                }
                // 维修累计
                Cell weiXiuSumCell = row.createCell(columnIndex.getAndIncrement());
                if (t.getWeiXiuSum() != null) {
                    weiXiuSumCell.setCellValue(t.getWeiXiuSum().doubleValue());
                }
                weiXiuSumCell.setCellStyle(numberCellStyle);
            }
            wb.write(os);
        }
    }
}
