package com.nokia.finance.tasks.pojo.vo.car_report;

import com.nokia.finance.tasks.config.CarReportConfig;
import com.nokia.finance.tasks.pojo.po.car_report.CarWgPo;
import lombok.Data;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.CellStyle;
import org.apache.poi.ss.usermodel.DataFormat;
import org.apache.poi.ss.usermodel.HorizontalAlignment;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.VerticalAlignment;
import org.apache.poi.xssf.streaming.SXSSFSheet;
import org.apache.poi.xssf.streaming.SXSSFWorkbook;

import java.io.IOException;
import java.io.OutputStream;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.util.Comparator;
import java.util.List;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.stream.Stream;

@Data
public class CarWgycVo {
    /**
     * 全省车辆违规行驶累计次数
     */
    private String qswg;
    /**
     * 违规车辆次数最多单位
     */
    private String wgzddw;
    /**
     * 违规最多次数
     */
    private String wgzdcs;
    /**
     * 违规用车一级单位统计
     */
    private List<CarWgUnitVo> wgUnitList;

    public CarWgycVo(List<CarWgUnitVo> wgUnitList, List<CarWgPo> wgList, CarReportConfig carReportConfig) throws IOException {
        this.wgUnitList = wgUnitList;
        wgUnitList.stream().filter(t -> t.getFirstUnit().equals("合计")).findFirst()
                .ifPresent(t -> this.qswg = t.getTotal());
        wgUnitList.stream().filter(t -> !t.getFirstUnit().equals("合计"))
                .max(Comparator.comparingInt(t -> Integer.parseInt(t.getTotal()))).ifPresent(t -> {
            this.wgzddw = t.getFirstUnit();
            this.wgzdcs = t.getTotal();
        });
        writeWgXlsx(wgList, carReportConfig);
    }

    /**
     * 违规用车明细xlsx
     */
    private void writeWgXlsx(List<CarWgPo> wgList, CarReportConfig carReportConfig) throws IOException {
        try (SXSSFWorkbook wb = new SXSSFWorkbook();
//             FastByteArrayOutputStream os = new FastByteArrayOutputStream()
             OutputStream os = Files.newOutputStream(Paths.get(carReportConfig.getWgXlsx()))
        ) {
            DataFormat dataFormat = wb.createDataFormat();
            // 日期时间样式
            CellStyle dateTimeCellStyle = wb.createCellStyle();
            dateTimeCellStyle.setDataFormat(dataFormat.getFormat("yyyy/MM/dd HH:mm:ss"));
            // 表头样式
            CellStyle headerCellStyle = wb.createCellStyle();
            headerCellStyle.setAlignment(HorizontalAlignment.CENTER);
            headerCellStyle.setVerticalAlignment(VerticalAlignment.CENTER);
            SXSSFSheet sheet = wb.createSheet();
            AtomicInteger rowIndex = new AtomicInteger(0);
            // 表头
            Row headerRow = sheet.createRow(rowIndex.getAndIncrement());
            List<String> headers = Stream.of("车辆所属单位", "车牌号码", "车辆类型", "车辆使用性质", "报警区域描述",
                    "报警规则描述", "越界时间", "是否执行中越界", "单位", "二级", "三级").toList();
            int headerSize = headers.size();
            for (int i = 0; i < headerSize; i++) {
                Cell cell = headerRow.createCell(i);
                cell.setCellValue(headers.get(i));
                cell.setCellStyle(headerCellStyle);
                // 根据内容长度设置列宽
                int columnWidth = headers.get(i).length() * 256 * 2 + 256 * 12;
                sheet.setColumnWidth(i, columnWidth);
            }
            for (CarWgPo t : wgList) {
                AtomicInteger columnIndex = new AtomicInteger(0);
                Row row = sheet.createRow(rowIndex.getAndIncrement());
                // 车辆所属单位
                Cell cheLiangSuoShuDanWeiCell = row.createCell(columnIndex.getAndIncrement());
                if (t.getCheLiangSuoShuDanWei() != null) {
                    cheLiangSuoShuDanWeiCell.setCellValue(t.getCheLiangSuoShuDanWei());
                }
                // 车牌号
                Cell chePaiHaoCell = row.createCell(columnIndex.getAndIncrement());
                if (t.getChePaiHao() != null) {
                    chePaiHaoCell.setCellValue(t.getChePaiHao());
                }
                // 车辆类型
                Cell cheLiangLeiXingCell = row.createCell(columnIndex.getAndIncrement());
                if (t.getCheLiangLeiXing() != null) {
                    cheLiangLeiXingCell.setCellValue(t.getCheLiangLeiXing());
                }
                // 车辆使用性质
                Cell cheLiangShiYongXingZhiCell = row.createCell(columnIndex.getAndIncrement());
                if (t.getCheLiangShiYongXingZhi() != null) {
                    cheLiangShiYongXingZhiCell.setCellValue(t.getCheLiangShiYongXingZhi());
                }
                // 报警区域描述
                Cell baoJingQuYuMiaoShuCell = row.createCell(columnIndex.getAndIncrement());
                if (t.getBaoJingQuYuMiaoShu() != null) {
                    baoJingQuYuMiaoShuCell.setCellValue(t.getBaoJingQuYuMiaoShu());
                }
                // 报警规则描述
                Cell baoJingGuiZeMiaoShuCell = row.createCell(columnIndex.getAndIncrement());
                if (t.getBaoJingGuiZeMiaoShu() != null) {
                    baoJingGuiZeMiaoShuCell.setCellValue(t.getBaoJingGuiZeMiaoShu());
                }
                // 越界时间
                Cell yueJieShiJianCell = row.createCell(columnIndex.getAndIncrement());
                if (t.getYueJieShiJian() != null) {
                    yueJieShiJianCell.setCellValue(t.getYueJieShiJian());
                }
                yueJieShiJianCell.setCellStyle(dateTimeCellStyle);
                // 是否执行中越界
                Cell shiFouZhiXingZhongYueJieCell = row.createCell(columnIndex.getAndIncrement());
                if (t.getShiFouZhiXingZhongYueJie() != null) {
                    shiFouZhiXingZhongYueJieCell.setCellValue(t.getShiFouZhiXingZhongYueJie());
                }
                // 单位
                Cell firstUnitCell = row.createCell(columnIndex.getAndIncrement());
                if (t.getFirstUnit() != null) {
                    firstUnitCell.setCellValue(t.getFirstUnit());
                }
                // 二级
                Cell secondUnitCell = row.createCell(columnIndex.getAndIncrement());
                if (t.getSecondUnit() != null) {
                    secondUnitCell.setCellValue(t.getSecondUnit());
                }
                // 三级
                Cell thirdUnitCell = row.createCell(columnIndex.getAndIncrement());
                if (t.getThirdUnit() != null) {
                    thirdUnitCell.setCellValue(t.getThirdUnit());
                }
            }
            wb.write(os);
        }
    }
}
