package com.nokia.finance.tasks.service.common;

import com.nokia.finance.tasks.dao.common.AreaDao;
import com.nokia.finance.tasks.pojo.po.common.AreaPo;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Slf4j
@Service
public class AreaService {
    private final AreaDao areaDao;

    public AreaService(AreaDao areaDao) {
        this.areaDao = areaDao;
    }

    /**
     * 查询地市
     * @return {@link List}<{@link AreaPo}>
     */
    public List<AreaPo> getCities() {
        return areaDao.getCities();
    }

    /**
     * 查询区县
     * @return {@link List}<{@link AreaPo}>
     */
    public List<AreaPo> getDistricts() {
        return areaDao.getDistricts();
    }

    /**
     * 获取地区字典，key：areaId，value：地区
     * @param cities 地市列表
     * @param districts 区县列表
     */
    public Map<String, AreaPo> getAreaMap(List<AreaPo> cities, List<AreaPo> districts) {
        Map<String, AreaPo> map = new HashMap<>();
        cities.forEach(t -> map.put(t.getAreaId(), t));
        districts.forEach(t -> map.put(t.getAreaId(), t));
        return map;
    }

    /**
     * 获取区县列表字典，key：地市id，value：区县列表
     * @param cities 地市列表
     * @param districts 区县列表
     */
    public Map<String, List<AreaPo>> getDistrictListMap(List<AreaPo> cities, List<AreaPo> districts) {
        Map<String, List<AreaPo>> map = new HashMap<>();
        for (AreaPo city : cities) {
            map.put(city.getAreaId(), new ArrayList<>());
            for (AreaPo district : districts) {
                if (city.getAreaId().equals(district.getParentId())) {
                    map.get(city.getAreaId()).add(district);
                }
            }
        }
        return map;
    }
}
